﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Collections.Generic;

namespace MSUS.Silverlight.Thmx2Theme.Controls.Extensions
{
    public static class ThmxStyle
    {
        public static readonly DependencyProperty StyleKeyProperty =
            DependencyProperty.RegisterAttached("StyleKey", typeof(string), typeof(ThmxStyle), new PropertyMetadata(null, new PropertyChangedCallback(OnStyleKeyPropertyChanged)));

        public static string GetStyleKey(DependencyObject obj)
        {
            return obj.GetValue(StyleKeyProperty) as string;
        }
        public static void SetStyleKey(DependencyObject obj, string value)
        {
            obj.SetValue(StyleKeyProperty, value);
        }

        static void OnStyleKeyPropertyChanged(DependencyObject sender, DependencyPropertyChangedEventArgs args)
        {
            FrameworkElement el = (FrameworkElement)sender;
            RoutedEventHandler handler = null;
            handler = (s, e) =>
            {
                el.Loaded -= handler;

                ThmxTheme theme = GetTheme(el);
                if (theme == null)
                    return;

                string styleKey = GetStyleKey(el);
                if (theme.Resources.Contains(styleKey))
                {
                    el.Style = theme.Resources[styleKey] as Style;
                }
            };
            el.Loaded += handler;
        }

        static ThmxTheme GetTheme(FrameworkElement element)
        {
            FrameworkElement parent = element.Parent as FrameworkElement;
            while (parent != null && !(parent is ThmxTheme))
            {
                parent = parent.Parent as FrameworkElement;
            }
            return parent as ThmxTheme;
        }
    }
}
