﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Threading;

namespace MSUS.Silverlight.Thmx2Theme.Controls
{
    /// <summary>
    /// Handles the generation of WebRequests for Uris using the "thmx://" scheme
    /// </summary>
    internal class ThmxLoader : IWebRequestCreate
    {
        public static readonly string THMX_SCHEME = "thmx";
        static ThmxLoader _loader;
        static Dispatcher _dispatcher;

        private ThmxLoader()
        {
        }

        /// <summary>
        /// Registers the "thmx://" scheme with the WebRequest class
        /// </summary>
        public static void Initialize(Dispatcher dispatcher)
        {
            _dispatcher = dispatcher;
            _loader = new ThmxLoader();
            WebRequest.RegisterPrefix(THMX_SCHEME, _loader);
        }

        #region IWebRequestCreate Members

        public WebRequest Create(Uri uri)
        {
            ThmxRequest req = new ThmxRequest(uri);
            
            EventHandler<PayloadEventArgs<Uri>> handler = null;
            EventHandler<PayloadEventArgs<Uri>> failHandler = null;

            handler = (s, e) => {
                req.ResponseRead -= handler;
                req.DownloadFailed -= failHandler;
                if (ThemeLoaded != null)
                {
                    _dispatcher.BeginInvoke(() =>
                    {
                        ThemeLoaded(this, e);
                    });
                }
            };
            req.ResponseRead += handler;

            failHandler = (s, e) =>
            {
                req.DownloadFailed -= failHandler;
                req.ResponseRead -= handler;
                if (ThemeLoadingFailed != null)
                {
                    _dispatcher.BeginInvoke(() =>
                    {
                        ThemeLoadingFailed(this, e);
                    });
                }
            };

            req.ResponseRead += handler;
            req.DownloadFailed += failHandler;

            return req;
        }

        #endregion

        /// <summary>
        /// Event raised when a theme has been loaded
        /// </summary>
        public static event EventHandler<PayloadEventArgs<Uri>> ThemeLoaded;

        /// <summary>
        /// Event raised when the download of a thmx file has failed
        /// </summary>
        public static event EventHandler<PayloadEventArgs<Uri>> ThemeLoadingFailed;
    }
}
