﻿using System;
using System.Net;
using System.Windows;
using System.IO;

namespace MSUS.Silverlight.Thmx2Theme.Controls
{
    /// <summary>
    /// WebResponse implementation handling responses from ThmxRequest
    /// </summary>
    public class ThmxResponse : WebResponse
    {
        private Uri _uri;
        private long _contentLength = 0;
        MemoryStream _stream;

        public ThmxResponse(Uri uri)
        {
            _uri = uri;
        }
        public ThmxResponse(Stream stream)
        {
            _stream = new MemoryStream();
            stream.CopyTo(_stream);
            _stream.Seek(0, SeekOrigin.Begin);
        }

        #region WebResponse overrides

        public override Stream GetResponseStream()
        {
            if (_stream != null)
            {
                return _stream;
            }

            Stream cacheStream = ThemeCacheManager.GetCachedTheme(_uri, out _contentLength);

            _stream = new MemoryStream((int)cacheStream.Length);
            cacheStream.CopyTo(_stream);
            _stream.Seek(0, SeekOrigin.Begin);
            cacheStream.Close();

            return _stream;
        }
        public override void Close()
        {
            _stream.Close();
        }

        public override long ContentLength
        {
            get
            {
                return _contentLength;
            }
        }
        public override string ContentType
        {
            get
            {
                return @"application/octet-stream";
            }
        }
        public override Uri ResponseUri
        {
            get { return _uri; }
        }

        #endregion
    }
}
